/**************************************************************************
*   Copyright (C) 2008 Daniel Nicoletti <dantti85-pk@yahoo.com.br>        *
*   Copyright (C) 2008 Dario Freddi <drf54321@gmail.com>                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "Authorizations.h"

#include <assert.h>

#include "AuthorizationsModel.h"

#include <QSortFilterProxyModel>

#include <KIcon>
#include <KDebug>

namespace PkKAuthorization {

class AuthorizationsHelper
{
  public:
    AuthorizationsHelper() : q(0) {}
    ~AuthorizationsHelper() { delete q; }
    Authorizations *q;
};

K_GLOBAL_STATIC(AuthorizationsHelper, s_globalAuthorizations)

Authorizations *Authorizations::instance()
{
    if (!s_globalAuthorizations->q) {
        new Authorizations;
    }

    return s_globalAuthorizations->q;
}

Authorizations::Authorizations(QWidget *parent)
: KDialog(parent)
{
    Q_ASSERT(!s_globalAuthorizations->q);
    s_globalAuthorizations->q = this;

    kDebug() << "Constructing PolicyKitKDE singleton";

    setupUi( mainWidget() );
    setWindowIcon( KIcon("object-locked") );
    m_proxyModel = new QSortFilterProxyModel(this);
    m_model = new AuthorizationsModel(treeView);
    m_proxyModel->setSourceModel(m_model);
    m_proxyModel->setFilterRole(AuthorizationsModel::IdRole);
    treeView->setModel(m_proxyModel);
    connect(treeView->selectionModel(), SIGNAL( currentChanged(const QModelIndex &, const QModelIndex &) ),
        this, SLOT( actionClicked(const QModelIndex &) ) );

    connect(searchLine, SIGNAL(textChanged(const QString&)), m_proxyModel, SLOT(setFilterRegExp(const QString&)));

    setButtons( KDialog::Close );
    setCaption( i18n("Authorizations") );
// delete splitter->widget(1);
    scrollArea->setWidget(m_pkKAction = new PkKAction);

    PkKSetContext();
    polkit_context_set_load_descriptions(m_pkContext);
//     pkContext = polkit_context_new();

    pkPFileEntry = new QList<PolKitPolicyFileEntry *>;
    UpdateActionTree();
}

void Authorizations::actionClicked(const QModelIndex &index)
{
    PolKitPolicyFileEntry *pfe = m_model->polkitFileEntry(index);
    if (pfe) {
        qDebug() << polkit_policy_file_entry_get_id(pfe);
        m_pkKAction->setPolKitPolicyFileEntry(pfe);
        m_currentActionId = polkit_policy_file_entry_get_id(pfe);
    }
}

Authorizations::~Authorizations()
{
}

static polkit_bool_t
buildActionList (PolKitPolicyCache */*policy_cache*/, PolKitPolicyFileEntry *pfe, void *user_data)
{
// funny looks like this makes the start up slow..
//     Q_UNUSED(policy_cache);

    kDebug() << "Adding an action";

    AuthorizationsModel* pfeList = (AuthorizationsModel*) user_data;

    pfeList->addEntry(pfe);

    // keep iterating
    return false;
}

void Authorizations::polkit_config_changed(PolKitContext */*pk_context*/, void *user_data)
{
    Authorizations *auth = (Authorizations *) user_data;
    auth->UpdateActionTree();
}

void Authorizations::watchActivatedContext(int fd)
{
    Q_ASSERT(m_watches.contains(fd));

//    kDebug() << "watchActivated" << fd;

    polkit_context_io_func (m_pkContext, fd);
}

void Authorizations::UpdateActionTree()
{
    pkPFileEntry->clear();
    m_model->clear();
    PolKitPolicyCache *pkPCache;
    pkPCache = polkit_context_get_policy_cache(m_pkContext);
    polkit_policy_cache_foreach(pkPCache, buildActionList, m_model);
}

void Authorizations::PkKSetContext()
{
// DBusError dbus_error;
        PolKitError *pk_error;

//         if (_singleton != NULL)
//                 return g_object_ref (_singleton);

        /* g_debug ("Constructing singleton"); */

//         _singleton = g_object_new (POLKIT_GNOME_TYPE_CONTEXT, NULL);

//         if ((_singleton->priv->system_bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, error)) == NULL) {
//                 goto error;
//         }

        m_pkContext = polkit_context_new ();
        // needed to know about config changes
        polkit_context_set_io_watch_functions (m_pkContext, add_context_io_watch, remove_context_io_watch);
        // needed to know about any change so we can update the ui
        polkit_context_set_config_changed (m_pkContext, polkit_config_changed, this);

// //         polkit_context_set_io_watch_functions (pkContext, io_add_watch, io_remove_watch);
// //         polkit_context_set_config_changed (_singleton->pk_context, pk_config_changed, _singleton);

        pk_error = NULL;
        if (!polkit_context_init(m_pkContext, &pk_error)) {
                qDebug() << "Failed to initialize PolicyKit context: " << polkit_error_get_error_message (pk_error);
// 		TODO Add a KMessageBox
//                 if (error != NULL) {
//                         *error = g_error_new_literal (POLKIT_GNOME_CONTEXT_ERROR,
//                                                       POLKIT_GNOME_CONTEXT_ERROR_FAILED,
//                                                       polkit_error_get_error_message (pk_error));
//                 }
                polkit_error_free (pk_error);

        }

        /* TODO FIXME: I'm pretty sure dbus-glib blows in a way that
        * we can't say we're interested in all signals from all
        * members on all interfaces for a given service... So we do
        * this..
        */

//         dbus_error_init (&dbus_error);
//
//         /* need to listen to NameOwnerChanged */
// 	dbus_bus_add_match (dbus_g_connection_get_connection (_singleton->priv->system_bus),
// 			    "type='signal'"
// 			    ",interface='"DBUS_INTERFACE_DBUS"'"
// 			    ",sender='"DBUS_SERVICE_DBUS"'"
// 			    ",member='NameOwnerChanged'",
// 			    &dbus_error);
//
//         if (dbus_error_is_set (&dbus_error)) {
//                 dbus_set_g_error (error, &dbus_error);
//                 dbus_error_free (&dbus_error);
//                 goto error;
//         }
//
//         /* need to listen to ConsoleKit signals */
// 	dbus_bus_add_match (dbus_g_connection_get_connection (_singleton->priv->system_bus),
// 			    "type='signal',sender='org.freedesktop.ConsoleKit'",
// 			    &dbus_error);
//
//         if (dbus_error_is_set (&dbus_error)) {
//                 dbus_set_g_error (error, &dbus_error);
//                 dbus_error_free (&dbus_error);
//                 goto error;
//         }
//
//         if (!dbus_connection_add_filter (dbus_g_connection_get_connection (_singleton->priv->system_bus),
//                                          _filter,
//                                          _singleton,
//                                          NULL)) {
//                 *error = g_error_new_literal (POLKIT_GNOME_CONTEXT_ERROR,
//                                               POLKIT_GNOME_CONTEXT_ERROR_FAILED,
//                                               "Cannot add D-Bus filter");
//                 goto error;
//         }
//
//         _singleton->pk_tracker = polkit_tracker_new ();
//         polkit_tracker_set_system_bus_connection (_singleton->pk_tracker,
//                                                   dbus_g_connection_get_connection (_singleton->priv->system_bus));
//         polkit_tracker_init (_singleton->pk_tracker);
//
//         return _singleton;
//
// error:
//         g_object_unref (_singleton);
//         return NULL;

}

int Authorizations::add_context_io_watch(PolKitContext* context, int fd)
{
    kDebug() << "add_watch" << context << fd;

    QSocketNotifier *notify = new QSocketNotifier(fd, QSocketNotifier::Read, Authorizations::instance());
    Authorizations::instance()->m_watches[fd] = notify;

    notify->connect(notify, SIGNAL(activated(int)), Authorizations::instance(), SLOT(watchActivatedContext(int)));

    return fd; // use simply the fd as the unique id for the watch
}


//----------------------------------------------------------------------------

void Authorizations::remove_context_io_watch(PolKitContext* context, int id)
{
    assert( id > 0 );
    kDebug() << "remove_watch" << context << id;
    if( !Authorizations::instance()->m_watches.contains(id))
        return; // policykit likes to do this more than once

    QSocketNotifier* notify = Authorizations::instance()->m_watches.take(id);
    notify->deleteLater();
    notify->setEnabled( false );
}

}

#include "Authorizations.moc"
