/*  This file is part of the KDE project
    Copyright (C) 2008 Dario Freddi <drf54321@gmail.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include "AuthorizationsModel.h"
#include <QStandardItem>

#include <KDebug>

namespace PkKAuthorization {

AuthorizationsModel::AuthorizationsModel(QObject* parent)
    : QStandardItemModel(parent)
{
}

AuthorizationsModel::~AuthorizationsModel()
{
}

void
AuthorizationsModel::addEntry(PolKitPolicyFileEntry* entry, QStandardItem *parent)
{
    if (!parent) {
        parent = invisibleRootItem();
    }
    //m_entries.append(entry);
    QString mypath = polkit_policy_file_entry_get_id(entry);
    QStringList ep = mypath.split('.');
    QString root = ep.at(0)+'.'+ep.at(1)+'.'+ep.at(2);

    QList<QStandardItem*> list = findItems(root);

    QStandardItem *ritm;

    if (list.isEmpty()) {
        ritm = new QStandardItem(root);
        ritm->setData(root, IdRole);
        ritm->setData(QVariant::fromValue(entry), PolkitEntryRole);
        ritm->setData(Group, EntryTypeRole);
        parent->appendRow(ritm);
    } else {
        ritm = list.at(0);
    }

    for (int i=3; i<ep.count(); ++i) {
        QStandardItem *previous = 0;

        // I know, this is nasty, but my time is short and this thing works :D

        int r = 0;

        while (true) {
            QStandardItem *child = ritm->child(r);

            if (!child) {
                break;
            }

            if (child->data(IdRole).toString() == root) {
                previous = child;
                break;
            }

            ++r;
        }

        if (!previous) {
            QStandardItem *itm = new QStandardItem(ep.at(i));
            itm->setData(QVariant::fromValue(entry), PolkitEntryRole);
            if (i == ep.count() - 1) {
                kDebug() << "Setting" << ep.at(i) << "as an action with id" << mypath;
                itm->setData(mypath, IdRole);
                itm->setData(Action, EntryTypeRole);
            } else {
                itm->setData(root, IdRole);
                itm->setData(Group, EntryTypeRole);
            }

            ritm->appendRow(itm);

            ritm = itm;
        } else {
            ritm = previous;
        }
    }

    emit reset();
    //kDebug() << "Added entry to tree:" << m_tree->tree()["org"]->tree();
}

PolKitPolicyFileEntry* AuthorizationsModel::polkitFileEntry(const QModelIndex &index)
{
    return index.data(PolkitEntryRole).value<PolKitPolicyFileEntry*>();
}

AuthorizationsModel::EntryType AuthorizationsModel::entryType(const QModelIndex &index)
{
    return (EntryType)index.data(EntryTypeRole).toInt();
}

QString AuthorizationsModel::polkitId(const QModelIndex &index) const
{
    return index.data(IdRole).toString();
}

}

#include "AuthorizationsModel.moc"
