/***************************************************************************
*   Copyright (C) 2008 by Daniel Nicoletti                                *
*   dantti85-pk@yahoo.com.br                                              *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <KToolInvocation>

#include <QDebug>

#include "PkKCommon.h"
#include "PkKPolkitClient.h"
#include "PkKStrings.h"
#include "PkKAction.h"

namespace PkKAuthorization {

PkKAction::PkKAction(QWidget *parent)
: QWidget(parent), m_pfe(0)
{
    setupUi(this);

    m_fontBold.setBold(true);
    m_fontNotBold.setBold(false);

    connect( vendorUrlKUL, SIGNAL( leftClickedUrl( const QString & ) ),
        SLOT( openUrl( const QString & ) ) );

    PolKitResult result;
    QList<PolKitResult> results = resultsOrdered();
    for (int i = 0; i < results.size(); i++) {
        result = results.at(i);
        // Adds all results
        allowAnyCB->addItem( PkKStrings::getPolKitResult(result), result);
        allowInactiveCB->addItem( PkKStrings::getPolKitResult(result), result);
        allowActiveCB->addItem( PkKStrings::getPolKitResult(result), result);
    }
    QStringList headers;
    headers << "Entity" << "Scope" << "Obtained." << "How" << "Constraints";
    m_explicitSIM.setHorizontalHeaderLabels(headers);
}

PkKAction::~PkKAction()
{
}

void PkKAction::setPolKitPolicyFileEntry(PolKitPolicyFileEntry *pfe)
{
// TODO see if the clear in the qlist will already delete these
//     if (m_pfe != 0) {
//         polkit_policy_file_entry_unref(m_pfe);
//     }

    m_pfe = pfe;

    // the pfe is just arriving
    modifyPB->setEnabled(false);

    //Action description
    QString actionId    = polkit_policy_file_entry_get_id(pfe);
    QString description = polkit_policy_file_entry_get_action_description(pfe);
    QString iconName    = polkit_policy_file_entry_get_action_icon_name(pfe);
    QString vendor      = polkit_policy_file_entry_get_action_vendor(pfe);
    QString vendorUrl   = polkit_policy_file_entry_get_action_vendor_url(pfe);

    identifierL->setText(actionId);
    descriptionL->setText(description);

    if ( vendorUrl.isEmpty() ) {
        vendorUrlKUL->setText( vendor );
    }
    else {
        vendorUrlKUL->setText( vendor );
        vendorUrlKUL->setUrl( vendorUrl );
    }

    if ( !iconName.isEmpty() ) {
        //TODO do something...
    }

    // IMPLICIT AUTHORIZATIONS
    // defaults are what is set now
    m_defaults              = polkit_policy_file_entry_get_default(pfe);
    m_defaultsAllowAny      = polkit_policy_default_get_allow_any (m_defaults);
    m_defaultsAllowInactive = polkit_policy_default_get_allow_inactive (m_defaults);
    m_defaultsAllowActive   = polkit_policy_default_get_allow_active (m_defaults);
    // factory are the values that came from the .policy file
    m_factory               = polkit_policy_file_entry_get_default_factory(pfe);
    m_factoryAllowAny       = polkit_policy_default_get_allow_any(m_factory);
    m_factoryAllowInactive  = polkit_policy_default_get_allow_inactive(m_factory);
    m_factoryAllowActive    = polkit_policy_default_get_allow_active(m_factory);

    if ( polkit_policy_default_equals(m_defaults, m_factory) )
        revertPB->setEnabled(false);
    else
        revertPB->setEnabled(true);

    PolKitResult result;
    QList<PolKitResult> results = resultsOrdered();
    for (int i = 0; i < results.size(); i++) {
        result = results.at(i);
        // Activate or deactivating bold
        if (result != m_factoryAllowAny)
            allowAnyCB->setItemData(i, m_fontBold, Qt::FontRole);
        else
            allowAnyCB->setItemData(i, m_fontNotBold, Qt::FontRole);

        if (result != m_factoryAllowInactive)
            allowInactiveCB->setItemData(i, m_fontBold, Qt::FontRole);
        else
            allowInactiveCB->setItemData(i, m_fontNotBold, Qt::FontRole);

        if (result != m_factoryAllowActive)
            allowActiveCB->setItemData(i, m_fontBold, Qt::FontRole);
        else
            allowActiveCB->setItemData(i, m_fontNotBold, Qt::FontRole);
    }

    allowAnyCB->setCurrentIndex( allowAnyCB->findData(m_defaultsAllowAny) );
    allowInactiveCB->setCurrentIndex( allowInactiveCB->findData(m_defaultsAllowInactive) );
    allowActiveCB->setCurrentIndex( allowActiveCB->findData(m_defaultsAllowActive) );

    // Explicit Authorizations
    PolKitCaller *self;
    DBusError dbus_error;
    dbus_error_init (&dbus_error);
//     self = polkit_tracker_get_caller_from_pid(pkgc->pk_tracker, getpid(), &dbus_error);

    PolKitAuthorizationDB *authdb;
//     authdb = polkit_context_get_authorization_db (pkgc->pk_context);
//     if (authdb != NULL && strlen (action_id) > 0 && self != NULL) {

    
}

void PkKAction::on_allowAnyCB_currentIndexChanged(int index)
{
    if (!m_pfe)
        return;
    if ( allowAnyCB->itemData(index) == m_factoryAllowAny) {
        allowAnyCB->setFont(m_fontNotBold);
        allowAnyL->setFont(m_fontNotBold);
    }
    else {
        allowAnyCB->setFont(m_fontBold);
        allowAnyL->setFont(m_fontBold);
    }
    checkDefaults();
    checkModified();
}

void PkKAction::on_allowInactiveCB_currentIndexChanged(int index)
{
    if (!m_pfe)
        return;
    if ( allowInactiveCB->itemData(index) == m_factoryAllowInactive) {
        allowInactiveCB->setFont(m_fontNotBold);
        allowInactiveL->setFont(m_fontNotBold);
    }
    else {
        allowInactiveCB->setFont(m_fontBold);
        allowInactiveL->setFont(m_fontBold);
    }
    checkDefaults();
    checkModified();
}

void PkKAction::on_allowActiveCB_currentIndexChanged(int index)
{
    if (!m_pfe)
        return;
    if ( allowActiveCB->itemData(index) == m_factoryAllowActive) {
        allowActiveCB->setFont(m_fontNotBold);
        allowActiveL->setFont(m_fontNotBold);
    }
    else {
        allowActiveCB->setFont(m_fontBold);
        allowActiveL->setFont(m_fontBold);
    }
    checkDefaults();
    checkModified();
}

void PkKAction::checkDefaults()
{
    if ( polkit_policy_default_equals(m_defaults, m_factory) ) {
        // IF they are iqual we still need to check if the user has the defaults
        if ( allowAnyCB->itemData( allowAnyCB->currentIndex() ) == m_factoryAllowAny &&
            allowInactiveCB->itemData( allowInactiveCB->currentIndex() ) == m_factoryAllowInactive &&
            allowActiveCB->itemData( allowActiveCB->currentIndex() ) == m_factoryAllowActive )

            revertPB->setEnabled(false);
        else
            revertPB->setEnabled(true);
    }
    else
        revertPB->setEnabled(true);
}

bool PkKAction::checkModified()
{
    // IF they are iqual we still need to check if the user has the defaults
    if ( allowAnyCB->itemData( allowAnyCB->currentIndex() ) == m_defaultsAllowAny &&
        allowInactiveCB->itemData( allowInactiveCB->currentIndex() ) == m_defaultsAllowInactive &&
        allowActiveCB->itemData( allowActiveCB->currentIndex() ) == m_defaultsAllowActive)

        modifyPB->setEnabled(false);
    else
        modifyPB->setEnabled(true);
    return modifyPB->isEnabled();
}

void PkKAction::on_modifyPB_clicked()
{
    if ( PkKPolkitClient::instance()->getAuth(AUTH_ACTION_MODIFY_DEFAULTS) ) {
        PolKitError *pk_error;
        PolKitPolicyDefault *new_defaults;

        PolKitResult allowAny      = (PolKitResult) allowAnyCB->itemData( allowAnyCB->currentIndex() ).toInt();
        PolKitResult allowInactive = (PolKitResult) allowInactiveCB->itemData( allowInactiveCB->currentIndex() ).toInt();
        PolKitResult allowActive   = (PolKitResult) allowActiveCB->itemData( allowActiveCB->currentIndex() ).toInt();

        new_defaults = polkit_policy_default_new();
        polkit_policy_default_set_allow_any      (new_defaults, allowAny);
        polkit_policy_default_set_allow_inactive (new_defaults, allowInactive);
        polkit_policy_default_set_allow_active   (new_defaults, allowActive);

        pk_error = NULL;
        polkit_policy_file_entry_debug(m_pfe);
        if (!polkit_policy_file_entry_set_default (m_pfe, new_defaults, &pk_error)) {
            qDebug() << "Error: code=" << polkit_error_get_error_code(pk_error)
                     << ": " << polkit_error_get_error_name(pk_error)
                     << ": " << polkit_error_get_error_message(pk_error);
            polkit_error_free (pk_error);
        }

        polkit_policy_default_unref (new_defaults);
    }
}

void PkKAction::on_revertPB_clicked()
{
    if ( polkit_policy_default_equals(m_defaults, m_factory) ) {
        // IF They are equal but we got clicked
        // is because the user changed the values
        // lets select the default ones
        allowAnyCB->setCurrentIndex( allowAnyCB->findData(m_defaultsAllowAny) );
        allowInactiveCB->setCurrentIndex( allowInactiveCB->findData(m_defaultsAllowInactive) );
        allowActiveCB->setCurrentIndex( allowActiveCB->findData(m_defaultsAllowActive) );
    }
    else if ( PkKPolkitClient::instance()->getAuth(AUTH_ACTION_MODIFY_DEFAULTS) ) {
        // if auth is fine just try to change
        PolKitError *pk_error = NULL;
        if (!polkit_policy_file_entry_set_default (m_pfe, m_factory, &pk_error)) {
                qDebug() << "Error: code=" << polkit_error_get_error_code(pk_error)
                         << ": "           << polkit_error_get_error_name(pk_error)
                         << ": "           << polkit_error_get_error_message(pk_error);
                polkit_error_free (pk_error);
        }
    }
}

void PkKAction::openUrl(const QString &url)
{
    KToolInvocation::invokeBrowser(url);
}

QList<PolKitResult> PkKAction::resultsOrdered()
{
    return QList<PolKitResult>() << POLKIT_RESULT_NO
          << POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT
          << POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH
          << POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION
          << POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_ALWAYS
          << POLKIT_RESULT_ONLY_VIA_SELF_AUTH_ONE_SHOT
          << POLKIT_RESULT_ONLY_VIA_SELF_AUTH
          << POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_SESSION
          << POLKIT_RESULT_ONLY_VIA_SELF_AUTH_KEEP_ALWAYS
          << POLKIT_RESULT_YES;
}


}

#include "PkKAction.moc"
